Package: geos
Title: Open Source Geometry Engine ('GEOS') R API
Version: 0.2.4
Authors@R: 
    c(
      person(given = "Dewey",
             family = "Dunnington",
             role = c("aut", "cre"),
             email = "dewey@fishandwhistle.net",
             comment = c(ORCID = "0000-0002-9415-4582")),
       person(given = "Edzer",
              family = "Pebesma",
              role = c("aut"),
              email = "edzer.pebesma@uni-muenster.de",
              comment = c(ORCID = "0000-0001-8049-7069"))
    )
Description: Provides an R API to the Open Source Geometry Engine
  ('GEOS') library (<https://libgeos.org/>) and a vector format 
  with which to efficiently store 'GEOS' geometries. High-performance functions 
  to extract information from, calculate relationships between, and
  transform geometries are provided. Finally, facilities to import 
  and export geometry vectors to other spatial formats are provided.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.3
Suggests: testthat (>= 3.0.0), vctrs, sf
Imports: libgeos (>= 3.8.1-4), wk (>= 0.4.1)
URL: https://paleolimbot.github.io/geos/,
        https://github.com/paleolimbot/geos/
BugReports: https://github.com/paleolimbot/geos/issues
LinkingTo: libgeos, wk
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2023-11-30 02:22:08 UTC; deweydunnington
Author: Dewey Dunnington [aut, cre] (<https://orcid.org/0000-0002-9415-4582>),
  Edzer Pebesma [aut] (<https://orcid.org/0000-0001-8049-7069>)
Maintainer: Dewey Dunnington <dewey@fishandwhistle.net>
Repository: CRAN
Date/Publication: 2023-11-30 05:50:03 UTC
Built: R 4.4.0; x86_64-apple-darwin20.6.0; 2024-05-04 22:39:38 UTC; unix
