# Generated by roxygen2: do not edit by hand

S3method("[",geos_geometry)
S3method("[<-",geos_geometry)
S3method("[[",geos_geometry)
S3method("[[<-",geos_geometry)
S3method(as.character,geos_geometry)
S3method(as.data.frame,geos_geometry)
S3method(as_geos_geometry,WKB)
S3method(as_geos_geometry,blob)
S3method(as_geos_geometry,character)
S3method(as_geos_geometry,default)
S3method(as_geos_geometry,geos_geometry)
S3method(as_geos_geometry,wk_xy)
S3method(as_geos_geometry,wk_xyz)
S3method(as_geos_strtree,default)
S3method(as_geos_strtree,geos_geometry)
S3method(as_geos_strtree,geos_strtree)
S3method(as_geos_type_id,character)
S3method(as_geos_type_id,default)
S3method(as_geos_type_id,numeric)
S3method(as_wkb,geos_geometry)
S3method(as_wkt,geos_geometry)
S3method(as_xy,geos_geometry)
S3method(c,geos_geometry)
S3method(format,geos_geometry)
S3method(format,geos_strtree)
S3method(is.na,geos_geometry)
S3method(plot,geos_geometry)
S3method(print,geos_geometry)
S3method(print,geos_strtree)
S3method(rep,geos_geometry)
S3method(rep_len,geos_geometry)
S3method(str,geos_geometry)
S3method(str,geos_strtree)
S3method(vec_cast.geos_geometry,default)
S3method(vec_cast.geos_geometry,geos_geometry)
S3method(vec_cast.geos_geometry,wk_crc)
S3method(vec_cast.geos_geometry,wk_rct)
S3method(vec_cast.geos_geometry,wk_wkb)
S3method(vec_cast.geos_geometry,wk_wkt)
S3method(vec_cast.geos_geometry,wk_xy)
S3method(vec_cast.geos_geometry,wk_xyz)
S3method(vec_cast.wk_wkb,geos_geometry)
S3method(vec_cast.wk_wkt,geos_geometry)
S3method(vec_cast.wk_xy,geos_geometry)
S3method(vec_cast.wk_xyz,geos_geometry)
S3method(vec_ptype2.geos_geometry,default)
S3method(vec_ptype2.geos_geometry,geos_geometry)
S3method(vec_ptype2.geos_geometry,wk_crc)
S3method(vec_ptype2.geos_geometry,wk_rct)
S3method(vec_ptype2.geos_geometry,wk_wkb)
S3method(vec_ptype2.geos_geometry,wk_wkt)
S3method(vec_ptype2.geos_geometry,wk_xy)
S3method(vec_ptype2.geos_geometry,wk_xyz)
S3method(wk_crs,geos_geometry)
S3method(wk_crs,geos_strtree)
S3method(wk_handle,geos_geometry)
S3method(wk_set_crs,geos_geometry)
S3method(wk_writer,geos_geometry)
export(as_geos_geometry)
export(as_geos_strtree)
export(as_geos_type_id)
export(geos_area)
export(geos_basic_strtree)
export(geos_basic_strtree_finalized)
export(geos_basic_strtree_insert)
export(geos_basic_strtree_query)
export(geos_basic_strtree_query_filtered)
export(geos_basic_strtree_size)
export(geos_boundary)
export(geos_buffer)
export(geos_buffer_params)
export(geos_build_area)
export(geos_centroid)
export(geos_clearance_line_between)
export(geos_clip_by_rect)
export(geos_clone)
export(geos_concave_hull)
export(geos_concave_hull_of_polygons)
export(geos_constrained_delaunay_triangles)
export(geos_contains)
export(geos_contains_any)
export(geos_contains_matrix)
export(geos_contains_properly_any)
export(geos_contains_properly_matrix)
export(geos_convex_hull)
export(geos_coordinate_dimension)
export(geos_coverage_union)
export(geos_covered_by)
export(geos_covered_by_any)
export(geos_covered_by_matrix)
export(geos_covers)
export(geos_covers_any)
export(geos_covers_matrix)
export(geos_create_rectangle)
export(geos_crosses)
export(geos_crosses_any)
export(geos_crosses_matrix)
export(geos_delaunay_edges)
export(geos_delaunay_triangles)
export(geos_densify)
export(geos_difference)
export(geos_difference_prec)
export(geos_dimension)
export(geos_disjoint)
export(geos_disjoint_any)
export(geos_disjoint_matrix)
export(geos_distance)
export(geos_distance_frechet)
export(geos_distance_hausdorff)
export(geos_distance_indexed)
export(geos_empty)
export(geos_envelope)
export(geos_envelope_rct)
export(geos_equals)
export(geos_equals_any)
export(geos_equals_exact)
export(geos_equals_exact_any)
export(geos_equals_exact_matrix)
export(geos_equals_matrix)
export(geos_extent)
export(geos_geometry)
export(geos_geometry_n)
export(geos_geometry_writer)
export(geos_has_z)
export(geos_hilbert_code)
export(geos_inner_join)
export(geos_inner_join_keys)
export(geos_interpolate)
export(geos_interpolate_normalized)
export(geos_intersection)
export(geos_intersection_prec)
export(geos_intersects)
export(geos_intersects_any)
export(geos_intersects_matrix)
export(geos_is_clockwise)
export(geos_is_closed)
export(geos_is_empty)
export(geos_is_ring)
export(geos_is_simple)
export(geos_is_valid)
export(geos_is_valid_detail)
export(geos_is_within_distance)
export(geos_largest_empty_circle_spec)
export(geos_largest_empty_crc)
export(geos_length)
export(geos_line_merge)
export(geos_line_merge_directed)
export(geos_make_collection)
export(geos_make_linestring)
export(geos_make_point)
export(geos_make_polygon)
export(geos_make_valid)
export(geos_make_valid_params)
export(geos_maximum_inscribed_circle_spec)
export(geos_maximum_inscribed_crc)
export(geos_merge_lines)
export(geos_minimum_bounding_circle)
export(geos_minimum_bounding_crc)
export(geos_minimum_clearance)
export(geos_minimum_clearance_line)
export(geos_minimum_rotated_rectangle)
export(geos_minimum_width)
export(geos_nearest)
export(geos_nearest_frechet)
export(geos_nearest_hausdorff)
export(geos_nearest_indexed)
export(geos_node)
export(geos_normalize)
export(geos_num_coordinates)
export(geos_num_geometries)
export(geos_num_interior_rings)
export(geos_num_rings)
export(geos_offset_curve)
export(geos_orientation_index)
export(geos_overlaps)
export(geos_overlaps_any)
export(geos_overlaps_matrix)
export(geos_point_end)
export(geos_point_n)
export(geos_point_on_surface)
export(geos_point_start)
export(geos_polygon_hull_simplify)
export(geos_polygonize)
export(geos_polygonize_cut_edges)
export(geos_polygonize_full)
export(geos_polygonize_valid)
export(geos_precision)
export(geos_prepared_contains)
export(geos_prepared_contains_properly)
export(geos_prepared_covered_by)
export(geos_prepared_covers)
export(geos_prepared_crosses)
export(geos_prepared_disjoint)
export(geos_prepared_distance)
export(geos_prepared_intersects)
export(geos_prepared_is_within_distance)
export(geos_prepared_overlaps)
export(geos_prepared_touches)
export(geos_prepared_within)
export(geos_project)
export(geos_project_normalized)
export(geos_read_geojson)
export(geos_read_hex)
export(geos_read_wkb)
export(geos_read_wkt)
export(geos_read_xy)
export(geos_relate)
export(geos_relate_pattern)
export(geos_relate_pattern_create)
export(geos_relate_pattern_match)
export(geos_remove_repeated_points)
export(geos_reverse)
export(geos_ring_n)
export(geos_segment_intersection)
export(geos_set_precision)
export(geos_set_srid)
export(geos_shared_paths)
export(geos_simplify)
export(geos_simplify_preserve_topology)
export(geos_snap)
export(geos_srid)
export(geos_strtree)
export(geos_strtree_data)
export(geos_strtree_query)
export(geos_sym_difference)
export(geos_sym_difference_prec)
export(geos_touches)
export(geos_touches_any)
export(geos_touches_matrix)
export(geos_transform_xy)
export(geos_type)
export(geos_type_id)
export(geos_unary_union)
export(geos_unary_union_prec)
export(geos_union)
export(geos_union_prec)
export(geos_unique_points)
export(geos_unnest)
export(geos_version)
export(geos_voronoi_edges)
export(geos_voronoi_polygons)
export(geos_within)
export(geos_within_any)
export(geos_within_matrix)
export(geos_write_geojson)
export(geos_write_hex)
export(geos_write_wkb)
export(geos_write_wkt)
export(geos_write_xy)
export(geos_x)
export(geos_xmax)
export(geos_xmin)
export(geos_y)
export(geos_ymax)
export(geos_ymin)
export(geos_z)
export(vec_cast.geos_geometry)
export(vec_ptype2.geos_geometry)
importFrom(libgeos,libgeos_version)
importFrom(wk,as_wkb)
importFrom(wk,as_wkt)
importFrom(wk,as_xy)
importFrom(wk,vec_cast.wk_wkb)
importFrom(wk,vec_cast.wk_wkt)
importFrom(wk,vec_cast.wk_xy)
importFrom(wk,vec_cast.wk_xyz)
importFrom(wk,wk_crs)
importFrom(wk,wk_crs_output)
importFrom(wk,wk_handle)
importFrom(wk,wk_set_crs)
importFrom(wk,wk_writer)
useDynLib(geos, .registration = TRUE)
